using RimWorld;
using System.Collections.Generic;
using Verse;
using HarmonyLib;

namespace IdlePowerConsumption
{

    public class Facilities : IConsumer
    {
        private int updateRate;
        private int updateOffset;

        public Facilities(int updateR = 4)
        {
            updateRate = updateR;
            updateOffset = updateRate / 2;
        }

        public float RequiredPower(CompPowerTrader compPowerTrader, int updateTick)
        {
            float result = 0f;

            if((updateTick + updateOffset) % updateRate != 0)
            {
                result = -1f;
            }
            else if(compPowerTrader.parent.GetComp<CompFacility>() != null)
            {
                result = 10f;

                CompFacility facility = compPowerTrader.parent.GetComp<CompFacility>();
                List<Thing> linkedList = Traverse.Create(facility).Field("linkedBuildings").GetValue<List<Thing>>();
                foreach(Thing linked in linkedList)
                {
                    if(inUse(linked))
                    {
                        result = compPowerTrader.Props.basePowerConsumption;
                        break;
                    }
                }
            }

            return result;
        }

        public bool CheckComponents(CompPowerTrader compPowerTrader)
        {
            return false;
        }

        private bool inUse(Thing building)
        {
            bool result = false;

            // Vitals Monitor
            if(building is Building_Bed)
            {
                Building_Bed bed = building as Building_Bed;
                if(!bed.AnyUnoccupiedSleepingSlot)
                {
                    result = true;
                }
            }

            // Analyzer
            else if(building is Building_ResearchBench)
            {
                CompPowerTrader compPowerTrader = (building as ThingWithComps).GetComp<CompPowerTrader>();

                if(compPowerTrader.PowerOn
                    && compPowerTrader.PowerOutput == -compPowerTrader.Props.basePowerConsumption)
                {
                    result = true;
                }
            }

            return result;
        }
    }
}