using RimWorld;
using Verse;

namespace IdlePowerConsumption
{

    public class WithInterractionCell : IConsumer
    {
        private int updateRate;
        private int updateOffset;

        private Tickable tickable;

        public WithInterractionCell(int updateR = 4)
        {
            updateRate = updateR;
            updateOffset = updateRate / 2;

            tickable = new Tickable(updateR);
        }

        public float RequiredPower(CompPowerTrader compPowerTrader, int updateTick)
        {
            float result = 0f;

            if((updateTick + updateOffset) % updateRate != 0)
            {
                result = -1f;
            }
            else if(compPowerTrader.parent.def.hasInteractionCell && compPowerTrader.parent.def.tickerType == TickerType.Never)
            {
                result = 20f;

                foreach(Thing thing in compPowerTrader.parent.InteractionCell.GetThingList(compPowerTrader.parent.Map))
                {
                    Pawn pawn = thing as Pawn;
                    if(pawn == null || pawn.CurJob == null)
                    {
                        continue;
                    }

                    if(pawn.CurJob.targetA != null && pawn.CurJob.targetA == compPowerTrader.parent)
                    {

                        if(pawn.CurJob.def == JobDefOf.DoBill || pawn.CurJob.def == JobDefOf.Research || pawn.CurJob.def == JobDefOf.OperateDeepDrill)
                        {
                            result = compPowerTrader.Props.basePowerConsumption;
                            break;
                        }
                    }
                }
            }

            if(result == 20f)
            {
                result = tickable.RequiredPower(compPowerTrader, updateTick);
            }

            return result;
        }

        public bool CheckComponents(CompPowerTrader compPowerTrader)
        {
            return true;
        }
    }
}