using HarmonyLib;
using RimWorld;
using System.Collections.Generic;
using System.Reflection;
using Verse;

namespace IdlePowerConsumption
{

    [StaticConstructorOnStartup]
    static class IdlePowerConsumption
    {
        public static TickableConsumersRegistry tickableConsumersRegistry;

        static IdlePowerConsumption()
        {
            tickableConsumersRegistry = new TickableConsumersRegistry();
            Harmony harmony = new Harmony("sladki.rimworld.mod.idlepowerconsumption");
            harmony.PatchAll(Assembly.GetExecutingAssembly());
        }
    }

    //public float CurrentEnergyGainRate()
    [HarmonyPatch(typeof(PowerNet))]
    [HarmonyPatch("CurrentEnergyGainRate")]
    public static class CurrentEnergyGainRate_Patch
    {
        private static List<IConsumer> consumersList = new List<IConsumer>();
        private static List<IComponent> componentsList = new List<IComponent>();

        static CurrentEnergyGainRate_Patch()
        {
            consumersList.Add(new WithInterractionCell(15));
            consumersList.Add(new AutoDoor(20));
            consumersList.Add(new Facilities(30));
            componentsList.Add(new CompGlower());
        }

        private static int updateTick = 0;

        [HarmonyPrefix]
        public static bool CurrentEnergyGainRate_Prefix(PowerNet __instance)
        {
            // var watch = new Stopwatch();
            // watch.Start();

            foreach(CompPowerTrader compPowerTrader in __instance.powerComps)
            {
                if(compPowerTrader.PowerOn == false
                    || compPowerTrader.PowerOutput > 0)
                // || compPowerTrader.parent.GetComp<CompFlickable>() == null)
                {
                    continue;
                }

                float requiredPower = 0f;

                foreach(IConsumer consumer in consumersList)
                {
                    requiredPower = consumer.RequiredPower(compPowerTrader, updateTick);

                    if(requiredPower > 0f)
                    {
                        if(consumer.CheckComponents(compPowerTrader))
                        {
                            bool inUse = (requiredPower == compPowerTrader.Props.basePowerConsumption) ? true : false;
                            foreach(IComponent component in componentsList)
                            {
                                requiredPower = requiredPower + component.RequiredPower(compPowerTrader, inUse);
                            }
                        }

                        break;
                    }
                }

                if(requiredPower > 0)
                {
                    compPowerTrader.PowerOutput = -requiredPower;
                }
            }

            updateTick = (updateTick >= 60) ? 0 : updateTick + 1;

            return true;
        }
    }
}